#!/usr/bin/env python
import os
import argparse
import ExpEvalGen
import argparse

RING_EXP_EVAL = "ring_"


def gen_oss(template_path, oss_path, params):
    with open(oss_path, "w") as oss_f:
        for k in params.keys():
            oss_f.write("#define %s %s\n" % (k, str(params[k])))

        oss_f.write("#include \"%s\"\n" % template_path)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run smv/ltl rewriting benchmark.")

    parser.add_argument('-i', '--input-oss', type=str, required=True,
                      help="input oss model")
    parser.add_argument('-z', '--opt-ocra-bin', type=str, default=None,
                      help="path to opt-OCRA executable")
    parser.add_argument('-X', '--old-ocra-bin', type=str, required=True,
                      help="path to OCRA executable")
    parser.add_argument('-r', '--runlim-bin', type=str, required=True,
                      help="path to runlim executable")
    parser.add_argument('-c', '--cmd-file', type=str, required=True)
    parser.add_argument('-f', '--from-size', type=int, required=True)
    parser.add_argument('-t', '--to-size', type=int, required=True)
    parser.add_argument('-S', '--space-lim', type=int, default=1024, help="space in MB")
    parser.add_argument('-T', '--time-lim', type=int, default=3600, help="time in second")

    parser.add_argument('out', type=str)

    args = parser.parse_args()

    runlim_bin = args.runlim_bin
    res_path = args.out
    space_lim = args.space_lim * 1024
    time_lim = args.time_lim

    exp_eval_gen = ExpEvalGen.ExpEvalGenerator(
        run_cwd=args.out,
        name_prefix=RING_EXP_EVAL,
        gen_oss_func=gen_oss,
        running_cmd=args.cmd_file,
        template_oss=args.input_oss,
        runlim=args.runlim_bin)

    if args.opt_ocra_bin != None:
        exp_eval_gen.register_exec("opt_ocra", args.opt_ocra_bin)
    exp_eval_gen.register_exec("old_ocra", args.old_ocra_bin)

    param_list = [{'size' : val} for val in  range(args.from_size, args.to_size + 1)]


    exp_eval_gen.gen_all(param_list, tlim=time_lim, slim=space_lim)
