#!/usr/bin/env python
import os
import argparse
import ExpEvalGen
import argparse

RING_EXP_EVAL = "ring_"
DUMMY_OSS="dummy.oss"
CMD_TYPE=".cmd"
OSS_TYPE=".oss"


def gen_oss(template_path, oss_path, params):
    return params['OSS_FILE']


def collect_oss_files(search_path):
    return collect_files_with(search_path, OSS_TYPE)

def collect_cmd_files(search_path):
    return collect_files_with(search_path, CMD_TYPE)

def collect_files_with(path, pattern):
    return list(filter(lambda p: \
            os.path.isfile(os.path.join(path, p)) and pattern in p, \
            os.listdir(path)))

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run smv/ltl rewriting benchmark.")

    parser.add_argument('-z', '--opt-ocra-bin', type=str, required=True,
                      help="path to opt-OCRA executable")
    parser.add_argument('-x', '--new-ocra-bin', type=str, required=True,
                      help="path to new-OCRA executable")
    parser.add_argument('-X', '--old-ocra-bin', type=str, required=True,
                      help="path to OCRA executable")
    parser.add_argument('-r', '--runlim-bin', type=str, required=True,
                      help="path to runlim executable")
    parser.add_argument('-S', '--space-lim', type=int, default=1024, help="space in MB")
    parser.add_argument('-T', '--time-lim', type=int, default=3600, help="time in second")

    parser.add_argument('out', type=str)

    args = parser.parse_args()

    runlim_bin = args.runlim_bin
    res_path = args.out
    space_lim = args.space_lim * 1024
    time_lim = args.time_lim

    cwd_oss_files = [os.path.join(os.getcwd(), cwd_oss) for cwd_oss in collect_oss_files(os.getcwd())]
    print(cwd_oss_files)
    cwd_cmd_files = [os.path.join(os.getcwd(), cwd_cmd) for cwd_cmd in collect_cmd_files(os.getcwd())]

    print(cwd_cmd_files)
    param_list = [{'OSS_FILE' : cwd_oss} for cwd_oss in cwd_oss_files]

    print(param_list)

    for cmd_file in cwd_cmd_files:
        new_cwd = os.path.join(args.out,    \
                               os.path.basename(cmd_file).replace(CMD_TYPE, ""))
        print(new_cwd)

        os.mkdir(new_cwd)

        exp_eval_gen = ExpEvalGen.ExpEvalGenerator(
            run_cwd=new_cwd,
            name_prefix="DISCR",
            gen_oss_func=gen_oss,
            running_cmd=cmd_file,
            template_oss=os.path.join(os.getcwd(), DUMMY_OSS),
            runlim=args.runlim_bin)

        exp_eval_gen.register_exec("opt_ocra", args.opt_ocra_bin)
        exp_eval_gen.register_exec("smv_ocra", "%s -ocra-use-ltl2smv" % args.opt_ocra_bin)
        exp_eval_gen.register_exec("new_ocra", args.new_ocra_bin)
        exp_eval_gen.register_exec("old_ocra", args.old_ocra_bin)

        exp_eval_gen.gen_all(param_list, time_lim, space_lim)
