""" 2 minutes """
OVER_TIME = 2

""" 256 MB """
OVER_MEM = 256

SUBMIT = "submit.sh"

import os

class ExpEvalGenerator:
    def __init__(
        self,
        run_cwd,
        name_prefix,
        gen_oss_func,
        running_cmd,
        template_oss,
        runlim
    ):
        self.run_cwd = run_cwd
        self.name_prefix = name_prefix
        self.gen_oss = gen_oss_func
        self.cmd_f = running_cmd
        self.runlim_bin = runlim
        self.template_oss = template_oss
        self.execs = dict()


    def register_exec(self, exec_name, exec_path):
        self.execs[exec_name] = exec_path
        print(self.execs)

    def gen_all(self, param_list, tlim, slim):
        sh_list = []

        for executor in self.execs.keys():
            for param in param_list:
                job_name = "%s_%s_%s" % (os.path.basename(executor), \
                        self.name_prefix, self.param2dir(param))
                new_oss = "%s.oss" % self.param2dir(param)
                new_oss_path = os.path.join(self.run_cwd, new_oss)
                self.gen_oss(self.template_oss, new_oss_path, param)
                sh_i = self.gen_sh(self.execs[executor], executor, param, job_name, self.param2dir(param), \
                        new_oss_path, tlim, slim)

                sh_list.append(sh_i)


        self.gen_submit(sh_list, tlim, slim)

    def param2dir(self, param):
        return "-".join([str(elem) for elem in param.values()])

    def gen_sh(self, exe, exe_name, param, job_name, param_str,
            oss_file, tlim, slim):

        print("tlim=%d, slim=%s" % (tlim, slim))
        runlim_str = "%s -s %d -r %d -o %s_out" % \
            (self.runlim_bin, slim, tlim, os.path.join(self.run_cwd, job_name))

        ocra_str = "%s -source %s %s > %s_stdout" % (exe, self.cmd_f, oss_file, \
                os.path.join(self.run_cwd, job_name))



        sh_name = os.path.join(self.run_cwd, "%s_%s.sh" % \
                (exe_name, job_name))

        with open(sh_name, "w") as sh_f:
            sh_f.write("#!/usr/bin/bash\n")
            sh_f.write("%s %s" % (runlim_str, ocra_str))

        print("%s file written...\n" % sh_name)

        return sh_name

    def gen_submit(self, sh_list, tlim ,slim):
        min_tlim = tlim / 60 + OVER_TIME
        kslim = 1024 * (OVER_MEM + slim)

        submit = os.path.join(self.run_cwd, SUBMIT)
        with open(submit, "w") as s_f:
            s_f.write("#!/usr/bin/bash\n")
            i = 0
            for sh_i in sh_list:
                iid = "%s-%d" % (os.path.basename(self.run_cwd), i)
                s_f.write("sbatch --partition=%s --job-name=%s --cpus-per-task=1 --output=%s_out.txt --error=%s_err.txt --mem=%dK --time=%d --nice=20 --priority=2 %s\n" % ("identical-4", iid, iid, iid, kslim, min_tlim, sh_i))
                i += 1
