#!/usr/bin/env python
import os
import argparse

res_path = os.getcwd()
LTL = "ltl"
SMV = "smv"
runlim_bin = ""
space_lim = -1
time_lim = -1
ocra_bin = ""

DEF_OCRA_PATH = "ocra_release"
DEF_RUNLIM_PATH = "runlim"
DEF_CMD_FILE = "run.cmd"

def log(string):
    print(string)

def gen_sh(min_n, max_n, min_proc, max_proc, orig_oss_path, orig_cmd_path):
    sh_list = []

    for n in range(min_n, max_n+1):
        for proc in range(min_proc, max_proc+1):
            dir_name = "%d-%d_EXP" % (n, proc)
            new_dir = os.path.join(res_path, dir_name)
            os.mkdir(new_dir)

            log("Created new directory %s" % new_dir)

            oss_path = gen_oss(orig_oss_path, new_dir, n, proc)

            log("Generated oss file %s" % oss_path)

            shltl = os.path.join(res_path, "%s_%s.sh" % (dir_name, LTL))

            gen_runlim_str = "%s -s %d -r %d" % (runlim_bin, space_lim, time_lim)

            # Generate ltl sh file
            ltl_runlim_str = "%s -o %s_out" % (gen_runlim_str, shltl)

            with open(shltl, "w") as ltl_f:
                ltl_f.write("#!/usr/bin/bash\n")
                ltl_f.write("%s %s -source %s %s > %s_stdout" % (ltl_runlim_str, ocra_bin, orig_cmd_path, oss_path, shltl))

            log("Generated sh file %s" % shltl)
            # Generate smv sh file
            shsmv = os.path.join(res_path, "%s_%s.sh" % (dir_name, SMV))
            smv_runlim_str = "%s -o %s_out" % (gen_runlim_str, shsmv)

            with open(shsmv, "w") as smv_f:
                smv_f.write("#!/usr/bin/bash\n")
                smv_f.write("%s %s -ocra-use-ltl2smv -source %s %s > %s_stdout" % (smv_runlim_str, ocra_bin, orig_cmd_path, oss_path, shsmv))

            log("Generated sh file %s" % shsmv)
            sh_list.append(shltl)
            sh_list.append(shsmv)

    return sh_list

def gen_oss(oss_path, new_dir, n, proc):
    new_oss_path = os.path.join(new_dir, "%s_%s.oss" % (n, proc))

    oss_path = os.path.abspath(oss_path)
    with open(new_oss_path, "w") as oss_f:
        oss_f.write("#define n %d\n" % n)
        oss_f.write("#define m %d\n" % n)
        oss_f.write("#define g_n %d\n" % (n * proc))
        oss_f.write("#define g_m %d\n" % (n * proc))
        oss_f.write("#define size %d\n" % proc)
        oss_f.write("#include \"%s\"\n" % oss_path)

    return new_oss_path

def gen_submit(sh_list, tlim, slim):
    with open("submit.sh", "w") as s_h:
        s_h.write("#!/usr/bin/bash\n")
        i = 0
        for sh_i in sh_list:
            s_h.write("sbatch --partition=%s --job-name=%s-short-%d --cpus-per-task=1 --output=%d_out.txt --error=%d_err.txt --mem=%dM --time=%d --nice=20 --priority=2 %s\n" % ("identical-4", os.path.basename(res_path), i, i, i, slim+10, tlim / 60 + 2, sh_i))
            i += 1

if __name__ == "__main__":
    global ocra_bin
    global runlin_bin
    global res_path
    global space_lim
    global time_lim

    parser = argparse.ArgumentParser(description="Run smv/ltl rewriting benchmark.")

    parser.add_argument('-i', '--input-oss', type=str, required=True,
                      help="input oss model")
    parser.add_argument('-x', '--ocra-bin', type=str, default=DEF_OCRA_PATH,
                      help="path to OCRA executable")
    parser.add_argument('-r', '--runlim-bin', type=str, default=DEF_RUNLIM_PATH,
                      help="path to runlim executable")
    parser.add_argument('-c', '--cmd-file', type=str, default=DEF_CMD_FILE)
    parser.add_argument('-s', '--start-n', type=int, required=True)
    parser.add_argument('-e', '--end-n', type=int, required=True)
    parser.add_argument('-f', '--from-size', type=int, required=True)
    parser.add_argument('-t', '--to-size', type=int, required=True)
    parser.add_argument('-S', '--space-lim', type=int, default=1024, help="space in MB")
    parser.add_argument('-T', '--time-lim', type=int, default=3600, help="time in second")

    parser.add_argument('out', type=str)

    args = parser.parse_args()

    ocra_bin = args.ocra_bin
    runlim_bin = args.runlim_bin
    res_path = args.out
    space_lim = args.space_lim
    time_lim = args.time_lim

    sh_list = gen_sh(args.start_n, args.end_n, args.from_size, args.to_size, args.input_oss, args.cmd_file)

    gen_submit(sh_list, time_lim, space_lim)
