#!/usr/bin/env python
import ExpEvalGen
import subprocess
import argparse
import os

PT_EXP_EVAL = "pt_"
CMD_TYPE = ".cmd"
OSS_TYPE = ".oss"

def gen_oss(templ, new_oss, params):
    n = params["size"]

    with open(new_oss, "w") as n_f:
        n_f.write("#define size %d\n" % n)
        n_f.write("#include \"%s\"" % templ)

    return new_oss

def collect_cmd_files(search_path):
    return collect_files_with(search_path, CMD_TYPE)

def collect_oss_files(search_path):
    return collect_files_with(search_path, OSS_TYPE)

def collect_files_with(path, pattern):
    return list(filter(lambda p: \
            os.path.isfile(os.path.join(path, p)) and pattern in p, \
            os.listdir(path)))

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run old/new rewriting benchmark.")

    parser.add_argument('-i', '--inp-oss-folder', type=str, default=os.getcwd())
    parser.add_argument('-x', '--new-ocra-bin', type=str, default=None,
                      help="path to OCRA executable")
    parser.add_argument('-z', '--opt-ocra-bin', type=str, default=None,
                      help="path to OCRA executable")
    parser.add_argument('-X', '--old-ocra-bin', type=str, default=None,
                      help="path to old OCRA executable")
    parser.add_argument('-r', '--runlim-bin', type=str, required=True,
                      help="path to runlim executable")
    parser.add_argument('-s', '--start-n', type=int, required=True)
    parser.add_argument('-e', '--end-n', type=int, required=True)
    parser.add_argument('-S', '--space-lim', type=int, default=1024, help="space in MB")
    parser.add_argument('-T', '--time-lim', type=int, default=14400, help="time in second")
    parser.add_argument('--seed', type=int, default=0)
    parser.add_argument('out', type=str, help="Folder for the experiment")

    args = parser.parse_args()

    args.out = os.path.join(os.getcwd(), args.out)

    cwd_cmd_files = [os.path.join(os.getcwd(), cwd_cmd) for cwd_cmd in collect_cmd_files(os.getcwd())]
    cwd_oss_files = [os.path.join(args.inp_oss_folder, cwd_oss) for cwd_oss in collect_oss_files(args.inp_oss_folder)]

    print(cwd_cmd_files)
    param_list = [{'size' : val} for val in  range(args.start_n, args.end_n + 1)]

    for cmd_file in cwd_cmd_files:
        for oss_file in cwd_oss_files:
            new_cwd = os.path.join(args.out,    \
                                   "%s_%s" % (os.path.basename(cmd_file).replace(CMD_TYPE, ""),
                                              os.path.basename(oss_file).replace(OSS_TYPE, "")))
            print(oss_file)

            os.makedirs(new_cwd)

            exp_eval_gen = ExpEvalGen.ExpEvalGenerator(
                run_cwd=new_cwd,
                name_prefix=PT_EXP_EVAL,
                gen_oss_func=gen_oss,
                running_cmd=cmd_file,
                template_oss=oss_file,
                runlim=args.runlim_bin)

            if args.new_ocra_bin != None:
                exp_eval_gen.register_exec("new_ocra", args.new_ocra_bin)
            if args.old_ocra_bin != None:
                exp_eval_gen.register_exec("old_ocra", args.old_ocra_bin)

            if args.opt_ocra_bin != None:
                exp_eval_gen.register_exec("opt_ocra", args.opt_ocra_bin)
                exp_eval_gen.register_exec("smv_ocra", "%s -ocra-use-ltl2smv" % args.opt_ocra_bin)

            exp_eval_gen.gen_all(param_list, args.time_lim, args.space_lim)
