#!/usr/bin/python3

import matplotlib.pyplot as plt

from runlim2py import DATA_ROJS, DATA_FILE, DATA_TIME, DATA_SPACE, DATA_STATUS, \
    STATUS_TIMEOUT, STATUS_MEMOUT

def __plot(obj_vals_dict, axis_name, title,
           out_file, use_log_scale, only_scatter, timeout, order_y, incremental=False):

    fig, ax = plt.subplots()
    if use_log_scale:
        ax.set_yscale('log')

    print (obj_vals_dict)

    for name in obj_vals_dict:
        x = obj_vals_dict[name].keys()
        y = obj_vals_dict[name].values()

        if not order_y and not incremental:
            v1 = sorted(zip(x, y))
            x, y = zip(*v1)
        else:
            y = sorted(y)

            if incremental:
                for i in range(1, len(y)):
                    if y[i] == timeout:
                        y[i] = y[i-1]
                    else:
                        y[i] += y[i-1]

            x = range(len(y))

        ax.scatter(x, y, label=name)

        if not only_scatter:
            ax.plot(x, y)

    ax.grid()

    if not only_scatter and not incremental:
        ax.axhline(y=timeout, color='black', linestyle='-', label="timeout")

    ax.set_xlabel(axis_name[0])
    ax.set_ylabel(axis_name[1])
    ax.legend(loc='upper left')

    plt.title(title)
    fig.savefig(out_file)
    plt.show()

def scatter_plot(obj_vals0, obj_vals1, competitor0, competitor1, out_file):
    legend = { competitor0 : None, competitor1 : None}
    plt.axis("square")
    plt.xlabel("%s technique" % competitor0)
    plt.ylabel("%s technique" % competitor1)


    print ("OBJ_0: %s" % str(obj_vals0))
    print ("OBJ_1: %s" % str(obj_vals1))

    elem0 = []
    elem1 = []

    for el in obj_vals0:
        if obj_vals1.get(el) != None:
            elem0.append(obj_vals0[el])
            elem1.append(obj_vals1[el])

    plt.plot(elem0, elem1, 'x', color='blue')
    plt.legend(list(legend.values()), list(legend.keys()), loc=4)

    min_T = 0.01
    max_T = 14400 # TO

    plt.xlim(min_T, max_T)
    plt.ylim(min_T, max_T)
    plt.plot([min_T, max_T], [min_T, max_T], color='gray', ls='--')

    plt.xscale('log')
    plt.yscale('log')

    plt.grid(True)
    plt.savefig(out_file, bbox_inches='tight')
    plt.clf()

#scatter_plot({"a": 5 , "b" : , })
def single_param_plot(comps, axis_name, title, timeout, out_file):
    __plot(obj_vals_dict=comps,
            axis_name=axis_name,
            title=title,
            out_file=out_file,
            use_log_scale=True,
            only_scatter=False,
            order_y = False,
            timeout=timeout)

def gen_latex_table(comps, title, comp, params):
    print(comps)

def gen_obj_vals_from_runlim_obj(runlim_obj, name2par, name2comp, use_time, uplim):
    res = dict()

    for obj in runlim_obj[DATA_ROJS]:
        comp = name2comp(obj[DATA_FILE])
        comp_res = res.get(comp)

        if comp_res == None:
            comp_res = dict()
            res[comp] = comp_res

        par = name2par(obj[DATA_FILE])

        if comp_res.get(par) is not None:
            # TODO: shitty error handling
            print("Error: Multiple y values for the same x! par: %s" % par)
            print(obj)
            print(res)
            exit(1)

        print(obj)
        status = obj[DATA_STATUS]
        if use_time:
            if status == STATUS_TIMEOUT or obj[DATA_TIME] >= uplim:
                comp_res[par] = uplim
                comp_res.pop(par)
            elif obj[DATA_TIME] < 0.0001:
                comp_res[par] = 0.001
            else:
                comp_res[par] = obj[DATA_TIME]

            #print("comp_res: %s, obj_data: %s" % (str(comp_res[par]), str(obj[DATA_TIME])))
        else:
            if status == STATUS_MEMOUT or obj[DATA_SPACE] >= uplim:
                comp_res[par] = obj[DATA_SPACE]
            else:
                comp_res[par] = obj[DATA_SPACE]



    print(res)

    return res

def incremental_plot(comps, axis_name, title, timeout, out_file):
    __plot(obj_vals_dict=comps,
            axis_name=axis_name,
            title=title,
            out_file=out_file,
            use_log_scale=True,
            only_scatter=False,
            order_y = True,
            incremental=True,
            timeout=timeout)

def multi_param_unordered_plot(comps, axis_name, title, timeout, out_file):
    __plot(obj_vals_dict=comps,
            axis_name=axis_name,
            title=title,
            out_file=out_file,
            use_log_scale=True,
            only_scatter=False,
            order_y = True,
            timeout=timeout)
