#!/usr/bin/python3

import os
import argparse

DATA_ROJS = "runlim_objs"
DATA_NAME = "name"
DATA_TIME = "time"
DATA_SPACE = "space"
DATA_STATUS = "status"
DATA_RESULT = "result"
DATA_FILE = "file"

TOK_STATUS="status:"
TOK_OK = "ok"
TOK_TIMEOUT = "out of time"
TOK_MEMOUT = "out of memory"
TOK_SPACE = "space:"
TOK_TIME = "time:"
TOK_RTIME = "real:"
TOK_RESULT = "result:"
TOK_SIGKILL = "signal(9)"
TOK_SEGFAULT = "segmentation fault"
TOK_SAMPLE = "sample"

STATUS_OK = 0
STATUS_TIMEOUT = -1
STATUS_MEMOUT = -2
STATUS_SIGKILL = -3

TIME_TOK = "time:"

SAMPLE_POS = -1
SPACE_POS = -2
TIME_POS = -3
REAL_POS = -4
CHILD_POS = -5
RES_POS = -6
STAT_POS = -7

def str_to_time(t):
    # 8 = ' seconds'
    print(t.strip())

    res = float(t.strip()[:-8])
    print(res)

    return res

def str_to_space(s):
    # 3 = ' MB'
    return float(s.strip()[:-3])

def str_to_stat(s):
    stat = s.strip()

    if stat == TOK_TIMEOUT:
        return STATUS_TIMEOUT
    elif stat == TOK_MEMOUT:
        return STATUS_MEMOUT
    elif stat == TOK_OK:
        return STATUS_OK
    elif stat == TOK_SIGKILL:
        stat = STATUS_SIGKILL
    elif stat == TOK_SEGFAULT:
        print("Warning: segfaults in results!!")
        stat = STATUS_MEMOUT
    else:
        print("Invalid status: %s" % stat)
        assert(False)

def get_param(string, tok):
    str_div = string.split(tok)


    if len(str_div) != 2:
        print(tok)
        print(string)
        print(str_div)
        assert(False)

    if DATA_TIME in tok:
        print(str_div[1].strip())
    return str_div[1].strip()#replace("\t", "")

def read_runlim_file(runlim_file, #file2param, file2competitor
        #,params
        #,competitors
        ):
    assert(os.path.isfile(runlim_file))

    print("Reading runlim file: %s" % runlim_file)

    lines = []

    with open(runlim_file, "r") as r_f:
        lines = r_f.readlines()

    assert(len(lines) > -STAT_POS)

    res = dict()

    delta = 0
    while TOK_SAMPLE not in lines[delta-1]:
        delta = delta -1

    result = get_param(lines[RES_POS + delta], TOK_RESULT)
    res[DATA_RESULT] = int(result)

    time = get_param(lines[TIME_POS + delta], TOK_TIME)
    res[DATA_TIME] = str_to_time(time)

    space = get_param(lines[SPACE_POS + delta], TOK_SPACE)
    res[DATA_SPACE] = str_to_space(space)

    stat = get_param(lines[STAT_POS + delta], TOK_STATUS)
    res[DATA_STATUS] = str_to_stat(stat)

    res[DATA_FILE] = runlim_file

    #res[DATA_PARAMS] = file2param(runlim_file, params)

    #res[DATA_COMPETITORS] = file2competitor(runlim_file, competitors)

    return res

def is_runlim_file(f):
    return os.path.isfile(f) and "_out" in f and f[-4:] == "_out"
def runlim_folder2py(runlim_folder,
                     #file2param,
                     #file2competitor,
                     #competitors,
                     rec, final):

    print("Entering folder %s" % runlim_folder)

    for elem in [os.path.join(runlim_folder, path) for path in os.listdir(runlim_folder)]:
        if is_runlim_file(elem):
            robj = read_runlim_file(elem)#, file2param, file2competitor)

            #if robj[DATA_COMPETITOR] not in competitors:
            #    print("Error: competitior %s not part of the competitor list %s" %
            #            (robj[DATA_COMPETITOR], competitors))
            #    exit(1)

            final.append(robj)
        elif os.path.isdir(elem) and rec:
            runlim_folder2py(elem, rec, final)


def expeval2py(root_dir, exp_name,
        # competitors, f2p, f2c,
        rec,
        #params
        ):
    final = dict()

    final[DATA_NAME] = exp_name
    #final[DATA_COMPETITORS] = competitors
    #final[DATA_PARAMS] = params
    final[DATA_ROJS] = []

    runlim_folder2py(root_dir,
                     #f2p, f2c, competitors,
                     rec, final[DATA_ROJS])

    return final

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Generate runlimpyobj")

    parser.add_argument('-n', '--name', type=str, required=True)
    parser.add_argument('-i', '--inp-folder', type=str, required=True)
    parser.add_argument('-o', '--out-file', type=str, default="res.py")
    parser.add_argument('-r', '--recursive', action='store_true', default=False)
    # parser.add_argument('-p', '--params', nargs='?', required=True)
    # parser.add_argument('-c', '--competitors', nargs='?', required=True)

    args = parser.parse_args()

    eval_obj = expeval2py(root_dir=args.inp_folder,
            exp_name=args.name,
            #competitors=args.competitors,
            #f2p=myf2p, f2c=myf2c,
            rec=args.recursive
            #params=args.params
            )
    with open(args.out_file, "w") as o_f:
        o_f.write("%s" % str(eval_obj))
